//
//  ItemCell.swift
//  AutoResizeTableCell
//
//  Created by mw on 2016-07-25.
//  Copyright © 2016 mw. All rights reserved.
//

import UIKit

class ItemCell: UITableViewCell, UITextViewDelegate {
    
    @IBOutlet var textView: UITextView!
    @IBOutlet var heightConstraint: NSLayoutConstraint!

    var lastCalculatedTextHeight: CGFloat = 0.0
    var tableController: RefreshableTableViewController?
    var item: Item!     // must be class for reference semantics
    
    func useItem(item: Item) {
        self.item = item
        self.textView.text = item.text
        updateMeasurements()
    }
    
    func updateMeasurements() {
        let newHeight = calculateHeightTextView(textView, forText: textView.text)
        heightConstraint.constant = newHeight
    }

    func textViewDidChange(textView: UITextView) {
        // update the model with the current text
        item.text = textView.text
    }
    
    func textView(textView: UITextView, shouldChangeTextInRange range: NSRange, replacementText text: String) -> Bool {
        // it's important that we measure the text and set the constraint *before* the textview actually sets the new
        // text, else the scroll view will jitter and jump. that's why we catch "shouldChange" to get in ahead of that
        
        let newText = buildNewText(textView.text, range: range, replacementText: text)
        let newHeight = calculateHeightTextView(textView, forText: newText)
        refreshIfNeeded(newHeight)
        return true
    }
    
    func refreshIfNeeded(newHeight: CGFloat) {
        if newHeight != lastCalculatedTextHeight {
            lastCalculatedTextHeight = newHeight
            heightConstraint.constant = newHeight
            tableController?.refresh()
        }
    }
    
    func buildNewText(oldText: String, range: NSRange, replacementText: String) -> String {
        if let nsRange = oldText.rangeFromNSRange(range) {
            let newText = oldText.stringByReplacingCharactersInRange(nsRange, withString: replacementText)
            return newText
        }
        // if the range was invalid, just return the old text
        return oldText
    }
    
    func calculateHeightTextView(textView: UITextView, forText: String) -> CGFloat {
        
        if let font = textView.font {
            let padding = textView.textContainer.lineFragmentPadding
            let width = textView.contentSize.width - 2 * padding
            
            let attributes = [NSFontAttributeName: font]
            let newRect = forText.boundingRectWithSize(CGSize(width: width, height: CGFloat.max),
                                                       options: [.UsesLineFragmentOrigin, .UsesFontLeading],
                                                       attributes: attributes, context: nil)
            let textInset = textView.textContainerInset
            let newHeight = newRect.height + textInset.bottom + textInset.top
            
            return newHeight
        }

        // if we don't know what else to say, just return the old height, probably better than nothing
        NSLog("failed to calculate height, no font?")
        return textView.contentSize.height
    }
    
}
