//
//  MyTableViewController.swift
//  AutoResizeTableCell
//
//  Created by mw on 2016-07-25.
//  Copyright © 2016 mw. All rights reserved.
//

import UIKit

protocol RefreshableTableViewController {
    func refresh()
}

class MyTableViewController: UITableViewController, RefreshableTableViewController {
    var itemStore: ItemStore!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // move down the table view contents a bit
        
        let statusBarHeight = UIApplication.sharedApplication().statusBarFrame.height
        let insets = UIEdgeInsets(top: statusBarHeight, left: 0, bottom: 0, right: 0)
        tableView.contentInset = insets
        tableView.scrollIndicatorInsets = insets
        
        tableView.rowHeight = UITableViewAutomaticDimension
        tableView.estimatedRowHeight = 65
    }
    
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return itemStore.allItems.count
    }
    
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("ItemCell", forIndexPath: indexPath) as! ItemCell
        
        // cell needs a delegate link back to the tableView controller, so it can trigger refresh of row heights
        cell.tableController = self
        
        // hand the item to the cell. as it updates the item, the model store
        // is automatically kept up to date since it holds a reference to the item
        let item = itemStore.allItems[indexPath.row]
        cell.useItem(item)
        
        return cell
    }

    func refresh() {
        // after updating the size and constraints of the table view cell, the table itself
        // must refresh to adjust the distance between the rows
        
        tableView .beginUpdates()
        tableView.endUpdates()
    }
    
    override func didRotateFromInterfaceOrientation(fromInterfaceOrientation: UIInterfaceOrientation) {
        // updating all constraints and table rows after a rotation requires the heavy
        // hand of a full reload
        
        tableView.reloadData()
    }
    
}


